/*******************************************************************************
 * Copyright (c) 2010- Association for Decentralized Information Management in
 * Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.history;

import java.util.Arrays;

import org.simantics.databoard.binding.mutable.MutableVariant;
import org.simantics.databoard.binding.mutable.Variant;

/**
 * Sampling Configuration is a description how values are collected from
 * a valuation source and written into a history container.
 * SamplingConfiguration is an input file to SampleCollector.
 * 
 *  type SamplingConfiguration = {
 *          subscription : Subscription
 *       }
 *
 *  type Subscription = {
 *          variableId : Variant,
 *          deadband : Optional( Double ),
 *          interval : Optional( Variant )
 *       }
 * 
 * See <a href="https://www.simantics.org/wiki/index.php/Databoard_Specification#SampleCollector">Sample Collector</a>
 * @author Toni Kalajainen <toni.kalajainen@vtt.fi>
 */
public class SamplingConfiguration {

	public MutableVariant timeId;
	public Subscription[] subscriptions;
	
	/**
	 * Add a subscription to a node.
	 * 
	 * @param nodeId node id
	 * @param interval interval, optional
	 * @param deadband interval, optional
	 */
	public void addSubscription(Variant nodeId, Variant interval, Variant deadband) {
		Subscription s = new Subscription(nodeId, deadband, interval);
		
		if (subscriptions==null) {
			subscriptions = new Subscription[] { s }; 
		} else {
			subscriptions = Arrays.copyOf(subscriptions, subscriptions.length + 1, Subscription[].class);
			subscriptions[ subscriptions.length-1 ] = s;			
		}
	}
	
	public void setTimeVariable(Variant timeId) {
		this.timeId = new MutableVariant(timeId);
	}
	
}

