/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.type;

import java.util.Set;

import org.simantics.databoard.util.IdentityPair;

public class OptionalType extends DataType {
    public DataType componentType;
    
    public OptionalType() {    	
    }
    
    public OptionalType(DataType componentType) {
    	this.componentType = componentType;
    }
    
    public DataType getComponentType() {
    	return componentType;
    }
    
	@Override
	protected void collectSubtypes(Set<DataType> subtypes,
		Set<DataType> recursiveSubtypes) {
		componentType.collectSubtypes(subtypes, recursiveSubtypes);
	}
    
    @Override
    public int hashCode() {
    	return componentType.hashCode()+1;
    }
    
    @Override
    protected boolean deepEquals(Object obj,
    		Set<IdentityPair<DataType, DataType>> compareHistory) {
		if (this==obj) return true;
		if (obj instanceof OptionalType == false) return false;
		OptionalType other = (OptionalType) obj;
		return componentType.deepEquals(other.componentType, compareHistory);
    }     

	@Override
	public void accept(Visitor1 v, Object obj) {
	    v.visit(this, obj);        
	}

	@Override
	public <T> T accept(Visitor<T> v) {
	    return v.visit(this);
	}	
    
}
